
# This is the grammar for grammar files.
#
# Which means you can use the Pattern returned by compile()'ing this file
# to parse this file.
#
# So... this file serves at the same time as both an example source text
# and the grammar to parse it.
#
# I realize that's possibly more confusing than helpful, so I'll include
# another simpler example elsewhere.  :)
#
# To "test" this file, run esrapy.py directly (i.e., "python esrapy.py").


# Comment-sucking optional white space with indentation:
__ = <([\r\n]*([ \t]+|[ \t]*#[^\n\r]*))*>

# Comment-sucking white space to (and including) end of line (or end of source):
end_line  = <([ \t]*(#[^\r\n]*)?([\n\r]|\Z))+>
# Optional version of same:
end_line0 = <([ \t]*(#[^\r\n]*)?([\n\r]|\Z))*>

# Variable name, and integer:
varname = <[a-zA-Z_]+[a-zA-Z_0-9]*>
int     = <[0-9]+>

base_sep = base:atom sep:(__ '/' __ _:atom)?

# Tightly grouped (atomic) expressions:

atom =   mod; base:base_sep __ op:<[?*+]>
     | range; base:base_sep __ '{' __ range:int/(__ ',' __){1,2} __ '}'
     |   var; varname
     |   key; '\' __ _:varname
     |subset; var:varname __ '@' __ limit:int
     | regex; expr:(`<([^>]*)>`@1 | <`([^`]*)`>@1) group:(__ '@' __ _:int)?
     |string; (<"([^"]*)">@1 | <'([^']*)'>@1)
     | '(' __ _:pattern __ ')'

# Sequence of atoms in series:

sequence = atom | seq; (name:(_:varname __ ':' __)? pat:atom)/__{2}

# Indented blocks:

block = sequence | block; ">>" __ _:sequence | indent; head:sequence __ ">>" __ body:sequence

# Alternatives separated by "|":

alts = block | set; (attrs:(_:(int|varname)/(__ ',' __){1,2} __ ';' __)?
                pat:block)/(__ '|' __){2}

pattern = alts

# And the top level operation, assigning a pattern to a variable:
assign = var:varname __ '=' __ val:pattern end_line

# The assign includes trailing white space, but we still need to allow for heading:
file = end_line0 _:assign*

